package projekt.enigma.model;

/**
 * In dieser Klasse wird die Hardware zusammengefasst.
 * Alle Komponenten der Enigma laufen hier zusammen.
 *
 * <p>
 * walzen : Walze[] :           Ist ein Array der drei eingebauten Walzen Objekte
 * steckbrett : Steckbrett :    Das Steckbrett mit den Verbindungen aus dem Codebuch
 * reflektor : Reflektor :      Dies ist das Objekt der Reflektors
 * nachricht : String :         Die verschluesselte Nachricht wird in diesem String zusammengesetzt und gespeichert
 * kenngruppe : String :        Frueher definierte die Kennzeichnung die Einheit welche diese Enigma benutzte.
 * Wir missbrauchen die Kenngruppe für einen Benutzernamen
 * </p>
 */
public class Hardware {

    //region Variablen
    /**
     * walzen: Walze-Array: erzeugt ein privates Array mit der die Anzahl der Walzen festgelegt wird.
     */
    private Walze[] walzen;

    /**
     * steckbrett: Steckbrett: erzeugt ein privates Steckbrett-Objekt
     */
    private Steckbrett steckbrett;

    /**
     * reflektor: Reflektor: erzeugt ein privates Reflektor-Objekt.
     */
    private Reflektor reflektor;
    //endregion

    //region Konstruktor

    /**
     * Im Konstruktor legen wir die Anzahl der Walzen fest, welche die Hardware aufnehmen kann.
     * Die Enigma I, welche hier nachgebildet werden soll, konnte drei Walzen aufnehmen.
     */
    public Hardware() {
        this.walzen = new Walze[3];
    }
    //endregion

    //region Funktionen & Methoden

    /**
     * Setzt eine Walze ein.
     *
     * @param walze    : int : Setzt die Walze mit der angegebenen Nummer ein.
     * @param walzenNr : int : Die Nummer der einzusetzenden Walze.
     * @param ringstellung : int : Setzt den Ring der Walze auf die angegebene Nummer.
     */
    public void setzeWalzenNr(int walze, int walzenNr, int ringstellung) {
        this.walzen[walze] = new Walze(walzenNr, ringstellung);
    }

    /**
     * Setzt den Notch der Walze.
     *
     * @param walze    : int : Position der Walze.
     * @param position : int : Umspringpunkt auf der Walze.
     */
    public void setzeRing(int walze, int position) {
        this.walzen[walze].setRingstellung(position);
    }

    /**
     * Setzt den Startwert (position) der Walze (walze).
     *
     * @param walze     : int : Nummer der Walze.
     * @param buchstabe : char : Buchstabe der zugewiesen werden soll.
     */
    public void setzePosition(int walze, char buchstabe) {
        this.walzen[walze].setPosition(buchstabe);
    }

    /**
     * Verschluesselt den uebergebenen Buchstaben und fuegt ihn der Nachricht hinzu.
     * Debug Ausgabe auf der Konsole wird zusaetzlich ausgegeben.
     *
     * @param buchstabe char : Der zu ver/entschluesselnde Buchstabe.
     * @return char : Der ver/entschluesselte Buchstabe.
     */
    public char codiere(char buchstabe) {
        new Fehler().debug(this.getClass().getName() + "." +
                new Throwable().getStackTrace()[0].getMethodName(), true);

        buchstabe = Character.toUpperCase(buchstabe);
        String debug = String.format("Positionen: W1:%s W2:%s W3:%s",
                this.walzen[0].getPosition(),
                this.walzen[1].getPosition(),
                this.walzen[2].getPosition());
        debug += "\nI\tS\t3\t2\t1\tR\t1\t2\t3\tS\n" + buchstabe;

        this.dreheWalzen(1);

        //Codiere Vorgang wird fuer jede Walze bis zum Reflektor mit der codiere Funktion durchgefuehrt.
        //Auf dem Rueckweg wird mit der codiere2 Funktion codiert.
        buchstabe = this.steckbrett.codiere(Character.toUpperCase(buchstabe));
        debug += "\t" + buchstabe;
        buchstabe = this.walzen[2].codiere(buchstabe);
        debug += "\t" + buchstabe;
        buchstabe = this.walzen[1].codiere(buchstabe);
        debug += "\t" + buchstabe;
        buchstabe = this.walzen[0].codiere(buchstabe);
        debug += "\t" + buchstabe;
        buchstabe = this.reflektor.codiere(buchstabe);
        debug += "\t" + buchstabe;
        buchstabe = this.walzen[0].codiere2(buchstabe);
        debug += "\t" + buchstabe;
        buchstabe = this.walzen[1].codiere2(buchstabe);
        debug += "\t" + buchstabe;
        buchstabe = this.walzen[2].codiere2(buchstabe);
        debug += "\t" + buchstabe;
        buchstabe = this.steckbrett.codiere(buchstabe);
        debug += "\t" + buchstabe;
        new Fehler().debug(debug, false);

        return buchstabe;
    }

    /**
     * Nach jeder Codierung eines Buchstabens muessen die Walzen gedreht werden.
     * Hier ueberpruefen wir, welche der Walzen gedreht werden muessen und stossen den Vorgang an.
     *
     * @param richtung : int : +1 fuer eine Vorwaertsdrehung und -1 fuer eine Rueckwaertsdrehung.
     */
    public void dreheWalzen(int richtung) {
        if (richtung > 0) {
            for (int i = 0; i < richtung; i++) {
                if (this.walzen[2].dreheWalze(1)) {
                    if (this.walzen[1].dreheWalze(1)) {
                        this.walzen[0].dreheWalze(1);
                    }
                }
            }
        } else {
            for (int i = 0; i > richtung; richtung++) {
                if (this.walzen[2].dreheWalze(-1)) {
                    if (this.walzen[1].dreheWalze(-1)) {
                        this.walzen[0].dreheWalze(-1);
                    }
                }
            }
        }
    }
    //endregion

    //region Setter

    /**
     * Setzt eine Walze ein und speichert das Objekt im Walzen Array.
     *
     * @param position     : int : Position der Walze.
     * @param walze        : int : Die Nummer der Walze.
     * @param ringstellung : int : Umsprungpunkt der Walze.
     */
    public void setWalzen(int position, int walze, int ringstellung) {
        this.walzen[position] = new Walze(walze, ringstellung);
    }

    /**
     * Gibt das Objekt des Steckbretts zurueck.
     *
     * @return steckbrett : Steckbrett-Objekt:  Das eingesetzte Steckbrett mit seinen Kabeln, wenn eingesteckt.
     */
    public Steckbrett getSteckbrett() {
        return steckbrett;
    }

    /**
     * Setzt das uebergebene Steckbrett Objekt in die Hardware ein.
     *
     * @param steckbrett : Steckbrett : Objekt des Steckbretts mit allen verbundenen Kabeln.
     */
    public void setSteckbrett(Steckbrett steckbrett) {
        this.steckbrett = steckbrett;
    }
    //endregion

    //region Getter

    /**
     * Setzt den Reflektor ein.
     * In der Enigma I, welche hier nachgebaut wird, gab es nur einen Reflektor.
     * Spaetere Versionen hatten zwei oder mehr, deshalb haben wir hier auch schon ein Objekt dafuer erstellt,
     * welches ausgetauscht werden kann.
     *
     * @param reflektor : Reflektor : Das Objekt des Reflektors.
     */
    public void setReflektor(Reflektor reflektor) {
        this.reflektor = reflektor;
    }

    /**
     * Gibt die verbauten Walzen Objekte zurück.
     *
     * @return walzen : Walze[] : Array mit den Walzen Objekten.
     */
    public Walze[] getWalzen() {
        return this.walzen;
    }
    //endregion

}
